/**
 * Post-install script to make the necessary modifications to Webtop config
 * files for the Precision web applications.
 */
function updateWebtopConfig()
{
    log( 3, "ncp_topoviz postInstall script updating Webtop configuration" );

	// Carry out Webtop tool cgi registration by modifying the Webtop cgi.group file.
	try
	{
		configureCgiGroup();
	}
	catch ( e )
    {
        log( 0, "Error making Precision Topoviz additions to the Webtop cgi.group file : " + e );
    }

	// Set the aelview.queries.enabled property in the Webtop server.init file to true.
	try
	{
		configureWebtopInit();
	}
	catch ( e )
	{
		log( 0, "Error making Precision Topoviz modifications to the Webtop server.init file : " + e );
	}

	// Add the Precision items to the Webtop alerts menu.
	try
	{
		modifyAlertsMenu();
	}
	catch ( e )
	{
		log( 0, "Error making Precision Topoviz additions to the Webtop alerts.nova file : " + e );
	}

	// Append the Topoviz relevant, locale specific, menu properties to the Webtop versions.
	try
	{
		addTopovizMenuProperties();
	}
	catch ( e )
	{
		log( 0, "Error making Precision Topoviz menu property additions to the Webtop dynamicmenus.properties files : " + e );
	}


}


function revertWebtopConfig()
{
	log( 2, "Precision Topoviz uninstall script updating Webtop configuration" );

	// Remove the Topoviz additions to the Webtop cgi.group file.
	try
	{
		removeCgiGroupItems();
	}
	catch ( e )
	{
		log( 0, "Error removing Precision Topoviz additions from the Webtop cgi.group file : " + e );
	}

	// Remove the Topoviz menu items from the Webtop alerts menu.
	try
	{
		removeTopovizMenuItems();
	}
	catch ( e )
	{
		log( 0, "Error removing Precision Topoviz additions from the Webtop alerts.nova file : " + e );
	}

	// Remove the Topoviz relevant, locale specific, menu properties from the Webtop versions.
	try
	{
		removeTopovizMenuProperties();
	}
	catch ( e )
	{
		log( 0, "Error removing Precision Topoviz menu property additions from the Webtop dynamicmenus.properties files : " + e );
	}

	log( 3, "Precision Topoviz uninstall script finished updating Webtop configuration" );
}


/**
 * Makes additions to the Webtop cgi.group file to register the tools to
 * launch Precision tools off the AEL.
 */
function configureCgiGroup()
{
	var cgiGrpFilename = getCgiGroupFilename();
    log( 3, "Precision Topoviz install checking Webtop cgi registration file " + cgiGrpFilename);

	// Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    // NOTE: This will not work with Mac!
    file.initWithPath( cgiGrpFilename );

    // Create an input stream to read in the file.
    var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();
	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
    	{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, adding in the Precision lines
			// if the file has not been modified already.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = false;
			do
			{
  				hasmore = inStream.readLine(line);
				if (line.value.indexOf("launch-rootcause-event") > -1)
				{
					alreadyModified = true;
				}
				else
				{
					if (line.value.indexOf('}') == -1)
					{
						newFileContents += line.value;
						newFileContents += getEndOfLineChars();
					}
					else
					{
						newFileContents += "cgi(name=\"launch-rootcause-event.cgi\",useSmartPageCommands=\"true\",acl=\"*\",pathType=\"relative\",fileName=\"launch-rootcause-event.cgi\")";
						newFileContents += getEndOfLineChars();
						newFileContents += "cgi(name=\"launch-suppressed-events.cgi\",useSmartPageCommands=\"true\",acl=\"*\",pathType=\"relative\",fileName=\"launch-suppressed-events.cgi\")";
						newFileContents += getEndOfLineChars();
						newFileContents += "}";
					}
				}

			} while(hasmore);

			if (alreadyModified ==false)
			{
				// Backup existing cgi.group to cgi.group.orig
    			backupFile( cgiGrpFilename );

				// Write out the new cgi.group file.
       			 writeToFile( cgiGrpFilename, newFileContents );

				log( 2, "cgi.group modified by Precision Topoviz install. Original version stored as cgi.group.orig");
			}
    	}
    	catch ( e )
    	{
        	log( 0, e );
    	}
    	finally
    	{
        	inStream.close();
    	}
	}

}


/**
 * Modifies the Webtop server.init file to ensure AEL view queries are enabled.
 */
function configureWebtopInit()
{
	var serverInitFilename = getServerInitFilename();
	log( 3, "Precision Topoviz install checking Webtop configuration file " + serverInitFilename);

	// Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    // NOTE: This will not work with Mac!
    file.initWithPath( serverInitFilename );

    // Create an input stream to read in the file.
    var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();
	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
    	{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, setting the 'aelview.queries.enabled'
			// property to true if it is currently false.
			var line = {}, hasmore;
			var newFileContents = "";
			var modified = false;
			var aelviewPropPresent = false;
			var aelviewPropName = "aelview.queries.enabled";
			do
			{
  				hasmore = inStream.readLine(line);

				// Look for the property at the start of the line to avoid changing comments.
				if (line.value.indexOf(aelviewPropName) == 0)
				{
					aelviewPropPresent = true;
					if (line.value.indexOf("false") > -1)
					{
						modified = true;
						var newLine = line.value.replace("false", "true");
						newFileContents += newLine;
						newFileContents += getEndOfLineChars();
					}
					else
					{
						break;
					}
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}

			} while(hasmore);

			if (aelviewPropPresent == false)
			{
				newFileContents += (aelviewPropName + ":true");
				newFileContents += getEndOfLineChars();
				modified = true;
			}

			if (modified == true)
			{
				// Backup existing server.init to server.init.orig
    			backupFile( serverInitFilename );

				// Write out the new server.init file.
       			 writeToFile( serverInitFilename, newFileContents );

				log( 2, "server.init modified by Precision Topoviz install. Original version stored as server.init.orig");
			}
    	}
    	catch ( e )
    	{
        	log( 0, e );
    	}
    	finally
    	{
        	inStream.close();
    	}
	}

}


/**
 * Makes additions to the Webtop alerts.nova file to launch the Precision
 * web applications.
 */
function modifyAlertsMenu()
{
	var alertsMenuFilename = getAlertsMenuFilename();
	log( 3, "Precision Web Applications install checking Webtop alerts menu file " + alertsMenuFilename);

	// Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    // NOTE: This will not work with Mac!
    file.initWithPath( alertsMenuFilename );

    // Create an input stream to read in the file.
    var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();
	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
    	{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, adding in the Precision lines
			// if the file has not been modified already.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = false;
			do
			{
  				hasmore = inStream.readLine(line);
				if (line.value.indexOf("Topoviz") > -1)
				{
					alreadyModified = true;
				}
				else
				{
					if (line.value.indexOf('}') == -1)
					{
						newFileContents += line.value;
						newFileContents += getEndOfLineChars();
					}
					else
					{
						newFileContents += "separator(acl=\"*\")";
                		newFileContents += getEndOfLineChars();
						newFileContents += "tool(name=\"Topoviz\", label=\"Find in Hop View\", mnemonic=\"\", shortcut=\"\", acl=\"*\")";
                		newFileContents += getEndOfLineChars();
						newFileContents += "tool(name=\"FindInNetworkView\", label=\"Find in Network View\", mnemonic=\"\", shortcut=\"\", acl=\"*\")";
                		newFileContents += getEndOfLineChars();
						newFileContents += "tool(name=\"ShowRootCause\", label=\"Show Root Cause\", mnemonic=\"\", shortcut=\"\", acl=\"*\")";
                		newFileContents += getEndOfLineChars();
						newFileContents += "tool(name=\"ShowSuppressedEvents\", label=\"Show Suppressed Events\", mnemonic=\"\", shortcut=\"\", acl=\"*\")";
						newFileContents += getEndOfLineChars();
						newFileContents += "}";
					}
				}

			} while(hasmore);

			if (alreadyModified ==false)
			{
				// Backup existing alerts.nova to alerts.nova.orig
    			backupFile( alertsMenuFilename );

				// Write out the new alerts.nova file.
       			 writeToFile( alertsMenuFilename, newFileContents );
				log( 2, "alerts.nova modified by Precision Web Applications install. Original version stored as alerts.nova.orig");
			}
    	}
    	catch ( e )
    	{
        	log( 0, e );
    	}
    	finally
    	{
        	inStream.close();
    	}
	}

}


/**
 * Removes the Topoviz items from the Webtop cgi.group file.
 */
function removeCgiGroupItems()
{
	var cgiGrpFilename = getCgiGroupFilename();

    log( 3, "Precision Topoviz uninstall checking Webtop cgi registration file " + cgiGrpFilename);

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	// NOTE: This will not work with Mac!
	file.initWithPath( cgiGrpFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, checking to see if the file
			// contains the Topoviz additions.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = true;

			do
			{
				hasmore = inStream.readLine(line);
				if ( (line.value.indexOf("launch-rootcause-event.cgi") > -1) ||
					(line.value.indexOf("launch-suppressed-events.cgi") > -1) )
				{
					alreadyModified = false;
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}


			} while(hasmore);

			if (alreadyModified == false)
			{
				// Backup existing cgi.group to cgi.group.orig
				backupFile( cgiGrpFilename );

				// Write out the alerts.nova file.
				writeToFile( cgiGrpFilename, newFileContents );

				log( 2, "Webtop cgi.group file modified by Precision Topoviz uninstall. Original version stored as cgi.group.orig");
			}
		}
		catch ( e )
		{
			log( 0, e );
		}
		finally
		{
			inStream.close();
		}
	}
}


/**
 * Removes the Topoviz items from the Webtop alerts.nova file.
 */
function removeTopovizMenuItems()
{
	var alertsMenuFilename = getAlertsMenuFilename();

	log( 2, "Precision Topoviz uninstall checking Webtop alerts menu file " + alertsMenuFilename);

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	// NOTE: This will not work with Mac!
	file.initWithPath( alertsMenuFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, checking to see if the file
			// contains the Topoviz additions.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = true;

			do
			{
				hasmore = inStream.readLine(line);
				if ( (line.value.indexOf("Topoviz") > -1) ||
					(line.value.indexOf("FindInNetworkView") > -1) ||
					(line.value.indexOf("ShowRootCause") > -1) ||
					(line.value.indexOf("ShowSuppressedEvents") > -1) )
				{
					alreadyModified = false;
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}


			} while(hasmore);

			if (alreadyModified == false)
			{
				// Backup existing alerts.nova to alerts.nova.orig
				backupFile( alertsMenuFilename );

				// Write out the alerts.nova file.
				writeToFile( alertsMenuFilename, newFileContents );

				log( 2, "Topoviz menu items removed by Precision Topoviz uninstall. Original version stored as alerts.nova.orig");
			}
		}
		catch ( e )
		{
			log( 0, e );
		}
		finally
		{
			inStream.close();
		}
	}
}


/**
 * Returns the filename, including the path, of the Webtop cgi.group file.
 */
function getCgiGroupFilename()
{
	var cgiGroupFilename = getNchome();

	if ( ncipackage.shortArch == "win32" )
	{
		cgiGroupFilename += "\\etc\\webtop\\cgi\\cgi.group";
	}
	else
	{
		cgiGroupFilename += "/etc/webtop/cgi/cgi.group";
	}

	return cgiGroupFilename;
}


/**
 * Returns the filename, including the path, of the Webtop alerts menu file.
 */
function getServerInitFilename()
{
	var serverInitFilename = getNchome();

	if ( ncipackage.shortArch == "win32" )
	{
		serverInitFilename += "\\etc\\webtop\\server.init";
	}
	else
	{
		serverInitFilename += "/etc/webtop/server.init";
	}

	return serverInitFilename;
}


/**
 * Returns the filename, including the path, of the Webtop alerts menu file.
 */
function getAlertsMenuFilename()
{
	var alertsMenuFilename = getNchome();

	if ( ncipackage.shortArch == "win32" )
	{
		alertsMenuFilename += "\\etc\\webtop\\configstore\\ncwMenus\\alerts.nova";
	}
	else
	{
		alertsMenuFilename += "/etc/webtop/configstore/ncwMenus/alerts.nova";
	}

	return alertsMenuFilename;
}


/**
 * Makes any relevant additions to the Webtop dynamicmenus.properties files to localise the Topoviz tool name.
 */
function addTopovizMenuProperties()
{
	log( 2, "Precision MIB Browser install checking Webtop dynamicmenus.properties files." );

	// Create a list of dynamicmenus.properties files which are waiting to go into webtop.
	var newMenuFiles = getDynamicMenuFilenames();

	for( var i = 0; i < newMenuFiles.length; i++ )
	{
		modifyDynamicMenusProperties( newMenuFiles[i] );
	}
	
}

/**
 * Removes the lines for localising the Topoviz name from dynamicmenus.properties.
 */
function removeTopovizMenuProperties()
{
	log( 2, "Precision Topoviz uninstall checking Webtop dynamicmenus.properties files." );
	
	// Create a list of dynamicmenus.properties files which have been modified by this install.
	var newMenuFiles = getDynamicMenuFilenames();

	for( var i = 0; i < newMenuFiles.length; i++ )
	{
		removeDynamicMenusProperties( newMenuFiles[i] );
	}
	
}

/**
 * Makes any relevant additions to the Webtop dynamicmenus.properties file to localise the Topoviz tool name.
 */
function modifyDynamicMenusProperties( filename )
{
	var propFilename = getWebtopLocaleDir() + filename;

	var openingComments = getOpeningComments();

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);
	
			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = false;

			do
			{
				hasmore = inStream.readLine(line);

				if (line.value.indexOf(openingComments) > -1 )
				{
					alreadyModified = true;
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
			
			

			} while(hasmore);

			if (alreadyModified == false)
			{
				// read in from precision file and copy over new lines.
				newFileContents += getEndOfLineChars();
				newFileContents += readDynamicMenusProperties( filename );

				// Backup existing dynamicmenus.properties file to dynamicmenus.properties.orig
				backupFile( propFilename );

				// Write out the new dynamicmenus.properties file.
				writeToFile( propFilename, newFileContents );

				log( 2, "Dynamicmenus.properties has been updated with new properties. Original version stored as dynamicmenus.properties.orig");
			}
		}
		catch ( e )
		{
			log( 2, "DynamicMenus.properties file doesn't exist for this locale: " + propFilename );
		}
		finally
		{
			inStream.close();
		}
	}
}

/**
 * Reads in the new properties from the locales staging area.
 */
function readDynamicMenusProperties( filename )
{
	var propFilename = getDynamicMenusDir() + filename;

	var fileContents = "";

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line.
			var line = {}, hasmore;
		
			do
			{
				hasmore = inStream.readLine(line);

				fileContents += line.value;
				fileContents += getEndOfLineChars();
		
			} while(hasmore);

		}
		catch ( e )
		{
			log( 0, "Can not read in DynamicMenus.properties file from: " + propFilename );
			log( 0, e );
		}
		finally
		{
			inStream.close();
		}
	}

	return fileContents;
}

/**
 * Removes any relevant lines from the Webtop dynamicmenus.properties files which contain localises tool name properties.
 */
function removeDynamicMenusProperties( filename )
{
	var propFilename = getWebtopLocaleDir() + filename;

	var openingComments = getOpeningComments();
	var closingComments = getClosingComments();

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);
	
			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = true;

			do
			{
				hasmore = inStream.readLine(line);

				if (line.value.indexOf(openingComments) > -1 )
				{
					alreadyModified = false;
	
					// remove this line and all other lines until "#END" string is found

					// Just do nothing, no need to add an empty line.
					//newFileContents += getEndOfLineChars();
								
					removeLines:
						do
						{
							hasmore = inStream.readLine(line);

							if (line.value.indexOf(closingComments) > -1 )
							{
								// Just do nothing, no need to add an empty line.
								//newFileContents += getEndOfLineChars();
								break removeLines;
							}
							else
							{
								// Just do nothing, no need to add an empty line.
								//newFileContents += getEndOfLineChars();
							}
				
						} while( hasmore )
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
			
			

			} while(hasmore);

			if (alreadyModified == false)
			{
				// Backup existing dynamicmenus.properties file to dynamicmenus.properties.orig
				backupFile( propFilename );

				// Write out the new dynamicmenus.properties file.
				writeToFile( propFilename, newFileContents );

				log( 2, "Topoviz menu properties removed by Precision Topoviz uninstall. Original version stored as dynamicmenus.properties.orig");
			}
		}
		catch ( e )
		{
			log( 2, "DynamicMenus.properties file doesn't exist for this locale: " + propFilename );
		}
		finally
		{
			inStream.close();
		}
	}
}

/**
 * Creates an array of filenames that are waiting to go into webtop. This list will need to be updated whenever a new locale file is created.
 */
function getDynamicMenuFilenames()
{
	// This could be written a bit more dynamically.
	if ( ncipackage.shortArch == "win32" )
	{
		var filenames = [ 
			"\\en\\dynamicmenus.properties",
			"\\en\\gb\\dynamicmenus.properties",
			"\\en\\us\\dynamicmenus.properties" ];
	}
	else
	{
		var filenames = [ 
			"/en/dynamicmenus.properties",
			"/en/gb/dynamicmenus.properties",
			"/en/us/dynamicmenus.properties" ];
	}

	return filenames;
}

/**
 * Gets the location of the locales staging area. This would be product specific.
 */
function getDynamicMenusDir()
{
	var dir = getNchome();

	if( ncipackage.shortArch == "win32" )
	{
		dir += "\\etc\\precision\\locale\\menus\\topoviz";
	}
	else
	{
		dir += "/etc/precision/locale/menus/topoviz";
	}

	return dir;
}

/**
 * Gets the location of the webtop locale files.
 */
function getWebtopLocaleDir()
{
	var dir = getNchome();

	if( ncipackage.shortArch == "win32" )
	{
		dir += "\\guifoundation\\webapps\\webtop\\WEB-INF\\classes\\locales";
	}
	else
	{
		dir += "/guifoundation/webapps/webtop/WEB-INF/classes/locales";
	}

	return dir;
}

/**
 * Gets the opening comments used in our locale files.
 */
function getOpeningComments()
{
	return "#Dynamic Menu Labels for ncp_topoviz tools";
}

/**
 * Gets the closing comments used in our locale files.
 */
function getClosingComments()
{
	return "#END";
}
